/*
    The contents of this file are subject to the Mozilla Public
    License Version 1.0 (the "License"); you may not use this file
    except in compliance with the License. You may obtain a copy of
    the License at http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS
    IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
    implied. See the License for the specific language governing
    rights and limitations under the License.

    Written by Oleg Subbotin ( oleg@cs.nmt.edu )

    Copyright(C) 1998, Oleg Subbotin.
*/

#include <qpainter.h>
#include <qlayout.h>
#include <qapp.h>
#include <qlabel.h>
#include <qlined.h>

#include "scope.h"
#define BORDER 5
#define HEIGHT 25
#define SHOW_PERIOD 100

#define global_font (QFont ("times", 12, QFont::Bold) )

//
// Creates the Scope widget
//

Scope::Scope( QWidget *parent, const char *name )
    : QWidget( parent, name )
{
    initMetaObject();

    QLabel *label;
    QPushButton *button;

    QHBoxLayout *top = new QHBoxLayout(this, 5);

    QVBoxLayout *labels = new QVBoxLayout();
    QVBoxLayout *combos = new QVBoxLayout();

    top->addLayout(labels);
    top->addLayout(combos);

 // Create a combo box & label
    label = new QLabel( this, "label1");
    label->setText("Voltage Scale:");
    label->setFont( global_font );
    label->setMinimumSize( label->sizeHint().width(), HEIGHT );
    labels->addWidget( label, 0, AlignLeft );

    QComboBox *voltage_scale = new QComboBox( TRUE, this, "comboBox" );
    voltage_scale->insertItem( " 0.1 V / Div" );
    voltage_scale->insertItem( " 0.5 V / Div" );
    voltage_scale->insertItem( " 1.0 V / Div" );
    voltage_scale->insertItem( " 2.0 V / Div" );
    voltage_scale->insertItem( " 5.0 V / Div" );
    voltage_scale->setCurrentItem(3);
    voltage_scale->setFont( global_font );
    voltage_scale->setFixedSize( voltage_scale->sizeHint().width(), HEIGHT );
    combos->addWidget( voltage_scale, 0 , AlignRight );

    label = new QLabel( this, "label2");
    label->setText("Input Range:");
    label->setFont( global_font );
    label->setMinimumSize( label->sizeHint().width(), HEIGHT );
    labels->addWidget( label, 0, AlignLeft );

    QComboBox *input_range = new QComboBox( TRUE, this, "comboBox2" );
    input_range->insertItem( " 2.5 V" );
    input_range->insertItem( "  5  V" );
    input_range->insertItem( " 10  V" );
    input_range->setCurrentItem(2);
    input_range->setFont( global_font );
    input_range->setFixedSize( input_range->sizeHint().width(), HEIGHT );
    combos->addWidget( input_range, 0, AlignRight );

    label = new QLabel( this, "label3");
    label->setText("Time Scale:");
    label->setFont( global_font );
    label->setFixedSize( label->sizeHint().width(), HEIGHT );
    labels->addWidget( label, 0, AlignLeft );

    QComboBox *time_scale = new QComboBox( TRUE, this, "comboBox3" );
    time_scale->insertItem( "  1samp / div" );
    time_scale->insertItem( "  5samp / div" );
    time_scale->insertItem( " 10samp / div" );
    time_scale->insertItem( " 25samp / div" );
    time_scale->setCurrentItem(2);
    time_scale->setFont( global_font );
    time_scale->setFixedSize( time_scale->sizeHint().width(), HEIGHT );
    combos->addWidget( time_scale, 0, AlignRight );
/*
    label = new QLabel( this, "label4");
    label->setText("Base period:");
    label->setFont( global_font );
    label->setFixedSize( label->sizeHint().width(), HEIGHT);
    labels->addWidget( label, 0, AlignLeft );

    temp_base = new QLineEdit(this);
    temp_base->setFrame(FALSE);
    temp_base->setFont( global_font );
    temp_base->setText("50");
    temp_base->setFixedSize(temp_base->sizeHint().width(), HEIGHT);
    combos->addWidget(temp_base);
*/
    combos->addStretch(10);

    labels->addStretch( 5 );
    button = new QPushButton( "Pause", this );
    button->setFont( global_font );
    button->setMinimumSize(button->sizeHint());
    labels->addWidget( button );

    labels->addStretch( 10 );
    quit = new QPushButton( "Quit", this );
    quit->setFont( global_font );
    quit->setMinimumSize(quit->sizeHint());
    labels->addWidget( quit );
    connect( quit, SIGNAL(clicked()), qApp, SLOT(quit()) );

    channels = new Channel(this, 0);
    channels->setMinimumSize(500, 300);
//    channels->setBackgroundMode(NoBackground);
    
    top->addWidget(channels, 10);

    connect(voltage_scale , SIGNAL(activated(int)), channels, 
	    SLOT(SetVoltageScale(int)) );

    connect( button, SIGNAL(clicked()), channels, SLOT(Pause()) );

    connect(input_range, SIGNAL(activated(int)), channels, 
	    SLOT(SetInputRange(int)) );

    connect(time_scale, SIGNAL(activated(int)), channels, 
	    SLOT(SetTimeScale(int)) );

//    connect(temp_base, SIGNAL(returnPressed()), this,
//	    SLOT(SetBasePeriod()));

    channels->show();

    top->activate();
}

Channel::Channel ( QWidget *parent, int fifo, const char *name)
  : QWidget( parent, name)
{
    initMetaObject();

    char device[32] = { 0, };
 
    voltage_scale = 20;
    input_range = 100;
    time_scale = 10;
    pause = 0;
    index = 0;
    
    sprintf( device, "/dev/rtf%d", fifo);

    for (int i=0;i<MAX;i++) 
	data[i]=0;

    if ((in_file = open(device, O_RDONLY|O_NONBLOCK)) < 0) {
      fprintf(stderr, "Error opening /dev/rtf0\n");
      exit(1);
    }

    myTimer = startTimer(SHOW_PERIOD);
}
  
//
// Handles timer events for the channels widget.
//

void Channel::paintEvent(class QPaintEvent *event )
{
    QPainter paint;
    paint.begin(this);
//    paint.setBackgroundColor(black);

    ClearDisplay(&paint);
    ShowSignal(&paint);
    
    paint.end();
}

void Channel::timerEvent(class QTimerEvent *event )
{
  GetNewData();
  repaint();
}

void Channel::Pause()
{
    if (pause)
	myTimer = startTimer(SHOW_PERIOD);
    else
	killTimer(myTimer);

    pause = !pause;
}
    
void Channel::SetVoltageScale(int scale)
{
  switch (scale)
    {
    case 0:
      voltage_scale = 1;
      break;
    case 1:
      voltage_scale = 5;
      break;
    case 2:
      voltage_scale = 10;
      break;
    case 3:
      voltage_scale = 20;
      break;
    case 4:
      voltage_scale = 50;
      break;
    }
}

void Channel::SetInputRange(int scale)
{
  switch(scale)
    {
    case 0:
      input_range = 25;
      break;
    case 1:
      input_range = 50;
      break;
    case 2:
      input_range = 100;
      break;
    }
}

void Channel::SetTimeScale(int scale)
{
  switch (scale)
    {
    case 0:
      time_scale = 1;
      break;
    case 1:
      time_scale = 5;
      break;
    case 2:
      time_scale = 10;
      break;
    case 3:
      time_scale = 25;
      break;
    }
}

void Channel::SetData(int* array, int count)
{
	for(int i=0; i<count; data[index]=array[i++])
		index = (index+1)%MAX;
}

void Channel::GetNewData()
{
	static int mux = 0;
	static inside = 0;
	int temp[18];
	if (inside) { printf("Error!\n"); return; }

	inside=1;
	while (read(in_file,temp,sizeof(int)*18)==sizeof(int)*18) {
		if (temp[1] == mux) mux=(mux+1)%4;
		else { printf ("Wrong mux received:%d expected:%d \n ", temp[1], mux);
			mux = (temp[1]+1)%4;
			}

		if (temp[0] == 0x4dac4dac) 
			SetData(&temp[17], 1);
		else 
			printf("Error\n");
	}
	inside = 0;
}

//
// Displays the data by drawing straight lines between points.
//

#define WIN_X  400
#define WIN_Y  400
#define GRIDS   10
#define OFFSET   3

void Channel::ShowSignal(QPainter *paint)
{
  int temp_index=(index+MAX+1)%MAX;
  double delta = (MAX)/(time_scale*10);
  double i=MAX+OFFSET;
  int middle = 2047;
  double scale = input_range/(voltage_scale*5.0);
  int temp_data;
  int count = 0;

  paint->setWindow( 0,0,MAX+2*OFFSET,4096);
  
  paint->setPen(blue);

    temp_data=(int)(data[temp_index]*scale);
    temp_data=(temp_data>=2047)?2047:temp_data;
    temp_data=(temp_data<=-2047)?-2047:temp_data;
    paint->moveTo((int)i, middle-temp_data);

    while( count++ <= 10*time_scale ) {
	temp_data=(int)(data[temp_index]*scale);
	temp_data=(temp_data>=2047)?2047:temp_data;
	temp_data=(temp_data<=-2047)?-2047:temp_data;
	paint->lineTo((int)i,middle-temp_data); 
	i-=delta;
	temp_index = (temp_index+MAX+1)%MAX;
    }
}


void Channel::ClearDisplay(QPainter *paint)
{
    int i, delta;

    paint->setWindow( 0, 0, WIN_X+2*OFFSET, WIN_Y+2*OFFSET );

    paint->setPen(green);
    for (i=0; i<=GRIDS; i++) {
	 delta = i*WIN_X/GRIDS;
         paint->drawLine (OFFSET+delta, OFFSET, OFFSET+delta, WIN_Y+OFFSET);
    }

    for (i=0; i<=GRIDS; i++) {
	 delta = i*WIN_Y/GRIDS;
	 if ( i==(GRIDS/2)) 
		paint->setPen(red);
         paint->drawLine (OFFSET, OFFSET+delta, WIN_X+OFFSET, OFFSET+delta);
	 if ( i==(GRIDS/2)) 
		paint->setPen(green);
    }
}
