/*======================================================================

    Configuration defs for daqcard700 PCMCIA real-time client driver.

    daqcard700-config.h v1.04     (known internally @JHCS as daq700pcs-config.h)
    
    9-30-1999

    The contents of this file are subject to the Mozilla Public
    License Version 1.0 (the "License"); you may not use this file
    except in compliance with the License. You may obtain a copy of
    the License at http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS
    IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
    implied. See the License for the specific language governing
    rights and limitations under the License.

    Rev 1.01 by Steve Rosenbluth, JHCS 9-18-98 (stever@silicon.net)
    Rev 1.02 by Steve, JHCS 3-12-1999
    Rev 1.03 by Steve, JHCS 8-30-1999
    Rev 1.04 by Steve, JHCS 9-30-1999

    Copyright(C) 1998,1999 Jim Henson's Creature Shop.

======================================================================*/
/*  epson dot-matrix printer condense code */

#ifndef _DAQ700_CONFIG_H_   /* include protection */
#define _DAQ700_CONFIG_H_

#define RESULT_FIFO_ID      0       /* rt fifo into which daq int handler puts results */
#define RESULT_FIFO_LEN     1024    /* length of results fifo */
#define CTRL_FIFO_ID        1         /* rt fifo for sending control messages to daq driver */
#define CTRL_FIFO_LEN       256       /* length of control fifo */
#define DIG_RESULT_FIFO_ID  2     /* rt fifo into which daq int handler puts digital input results */
#define DIG_RESULT_FIFO_LEN 64    /* length of digital input result fifo */
#define STOP_ACQUISITION    0   /* command flags sent to daq driver in control struct */
#define START_ACQUISITION   1
#define CONFIGURE_ACQUISITION 2
#define BURST_ACQUISITION   3
#define CONTIN_ACQUISITION  4
#define DIGITAL_OUTPUT      5
#define TEN_VOLT            0
#define FIVE_VOLT           1
#define TWO_VOLT            2
#ifndef DAQ_MAGIC
    #define DAQ_MAGIC 0x4DAC4DAC
#endif
#define RESULT_PACKET_SIZE  18 /*19 for testing w/ request ids only ?!? */ /* number of integers in a single packet written to rt-fifo */
#define PACKET_HEADER_SIZE  2
#define BANK_ID             1       /* offset into result packet where the bank stamp is found*/
#define CLR_2_LSB_INT32     0xFFFC /* AND mask to clear two low bits of an int32 */
#define MAX_SAMPLE_RATE_USEC 10     /* maximum supported conversion rate */
#define MAX_BANKS           4       /* max number of banks of 16 samples we can address*/ 
#define MAX_CHANNELS        16      /* can only sample 1-16 channels on this card */
#define BTC_NUMSYS          0       /* Binary Twos Complement. F800 to 0000 is -Vmin to 0V, 0000 to 7FF is 0V to +Vmax */
#define BOB_NUMSYS          1       /* Bipolar Offset Binary. 0 to 4095 is -Vmin to +Vmax */
#ifndef TRUE
    #define TRUE 1
#endif
#ifndef FALSE
    #define FALSE 0
#endif


struct ctl_msg_struct   /* message sent to driver through rt-fifo to change daq parameters */
{
    int command;    /* configure, start or stop conversions */
    int conversion_mode;  /* reset to default mode, continuous mode, burst mode, */
    int num_chan;   /* how many (of 16) channels to scan */
    int single_chan;    /* the channel to sample when in single chan mode */
    int num_adc_banks;  /* how many banks (of num_chan) to convert (4 max), using external multiplexer */
    int volt_range;    /* 10 volt, 5 volt. 2.5 volt. always single ended, for now */
    int result_numsys;  /* Binary Two's Complement, Bipolar Offset Binary */
    int error_correcting;   /* whether user wants to use hardware calibration constants on results: 1=yes, 0=no*/
    int timer_period; /* num microsecs between conversions */
    /* 8 bit value to be written to the digital output port. */
    unsigned char dio_output_val;     /*NOTE: only 6 MSBs get written in multi-bank modes, 2 LSBs reserved for multiplexing */
    int num_dig_in_banks;   /* number of banks of digital inputs to get (4 max) */
                                            /* whether to add extra delay in order to stop banks bug */    /*unsigned message_id; */    /* this message's unique ID (up to size of int) for tracking conversions */
};

#endif /* _DAQ700_CONFIG_H_ */
