/* file: daq-config.c */
/* this file sends configuration/action messages to the Daqcard-700 rt-driver via a real time fifo */
/* steve rosenbluth JHCS 9-98 */
/* compile with: gcc -I/usr/src/linux/include -O2 -o daq-config daq-config.c */
/*  epson dot-matrix printer condense code */

/*----------------------------------------------------------------
    This source file is subject to the Mozilla Public
    License Version 1.0 (the "License"); you may not use this file
    except in compliance with the License. You may obtain a copy of
    the License at http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS
    IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
    implied. See the License for the specific language governing
    rights and limitations under the License.
------------------------------------------------------------------*/

#include <stdio.h>                                                             /* for printf, */
#include <stdlib.h>                                                             /* for atoi */
#include <string.h>                                                            /* for strncmp */
#include <sys/types.h>                                                         /* for open, */
#include <sys/stat.h>                                                          /* for open, */
#include <fcntl.h>                                                             /* for open */
#include <unistd.h>                                                            /* for write */
/*#include </usr/src/rtl/include/rtl_fifo.h>*/ /* old real time defs */
#include <rtlinux/rtl_fifo.h> /* new rtlb15 real time defs */
#include "../daqcard700-config.h"                                    /* defs for talking to daq driver */


/****************************/
int main(int argc, char *argv[])
{
    char mode = '!';
    int fd_ctl;                                                                /* file descriptors for real time fifos */
    struct ctl_msg_struct ctl_msg;                                             /* for talking to daq driver through fifo */

    if (argc > 1)
    {
        sscanf(argv[1], "%c", &mode);                                          /* convert ascii hex port value to decimal */
        if (argc > 2)
        {
            if (mode == 'o')                                                   /* digital output */
                ctl_msg.dio_output_val = atoi(argv[2]);                        /* convert byte to output */
            else if (strncmp(argv[2], "10", 2) == 0)
                ctl_msg.volt_range = TEN_VOLT;
            else if (strncmp(argv[2], "5", 1) == 0)
                ctl_msg.volt_range = FIVE_VOLT;
            else if (strncmp(argv[2], "2.5", 3) == 0)
                ctl_msg.volt_range = TWO_VOLT;
        }

        if (CTRL_FIFO_ID == 1)
            if ((fd_ctl = open("/dev/rtf1", O_WRONLY)) < 0)            /* this device is for this proc to send commands in to rtproc */
            {                                                                  /* this (2nd) rt fifo is for writing control mssgs */
                perror("daq-config: Error opening /dev/rtf1");
                exit(1);
            }
    }
    else
    {
        printf("Usage: daq-config [c b g s o] [10 5 2.5 0-255]\nc: continuous acq\nb: burst acq\ng: 'go' sample\ns: stop sampling\no: digital output\n10,5,2.5: voltage range\n0-255: digital output value\n");
        exit(0);
    }

    if (mode == 'c')                                                           /* continuous acq */
    {
        /* now send message to start the periodic task */
        ctl_msg.command = CONFIGURE_ACQUISITION;
        ctl_msg.conversion_mode = CONTIN_ACQUISITION;
        ctl_msg.num_chan = 16;
        ctl_msg.single_chan = 0;
        ctl_msg.num_adc_banks = 4;
        ctl_msg.timer_period = 30;
        ctl_msg.result_numsys = BOB_NUMSYS;
        ctl_msg.error_correcting = FALSE;
        ctl_msg.num_dig_in_banks = 4;
    }
    else if (mode == 'b')                                                      /* burst acq */
    {
        ctl_msg.command = CONFIGURE_ACQUISITION;
        ctl_msg.conversion_mode = BURST_ACQUISITION;
        ctl_msg.num_chan = 16;
        ctl_msg.single_chan = 0;
        ctl_msg.num_adc_banks = 4;
        ctl_msg.timer_period = 30;
        ctl_msg.result_numsys = BOB_NUMSYS;
        ctl_msg.error_correcting = FALSE;
        ctl_msg.num_dig_in_banks = 4;
    }
    else if (mode == 'g')                                                      /* "go" start acq */
    {
        ctl_msg.command = START_ACQUISITION;
        ctl_msg.conversion_mode = START_ACQUISITION;
        /*ctl_msg.message_id = -1;*/
/*    ctl_msg.num_chan  = 0;
 * ctl_msg.single_chan = 0;
 * ctl_msg.num_adc_banks = 0;
 * ctl_msg.timer_period = 0; */
    }
    else if (mode == 's')                                                      /* stop acq */
    {
        ctl_msg.command = STOP_ACQUISITION;
        ctl_msg.conversion_mode = STOP_ACQUISITION;
        ctl_msg.num_chan = 0;
        ctl_msg.single_chan = 0;
        ctl_msg.num_adc_banks = 0;
        ctl_msg.timer_period = 0;
    }
    else if (mode == 'o')                                                      /* stop acq */
    {
        ctl_msg.command = DIGITAL_OUTPUT;
        /* value is already in ctl_msg.dio_output_val */
    }

    /*printf("message sent is %d bytes\n",sizeof(ctl_msg));*/
    if (write(fd_ctl, &ctl_msg, sizeof(ctl_msg)) < 0)                          /* write startup message to control fifo file descriptor */
    {
        fprintf(stderr, "Can't send a command to RT-task\n");
        exit(1);
    }

    /* printf("\nSent message to daqcard700 ...\ncommand: %d, conversion_mode: %d, num_chan: %d, single_chan: %d, num_adc_banks: %d,
     * timer_period: %d\n",
     * ctl_msg.command,ctl_msg.conversion_mode,ctl_msg.num_chan,ctl_msg.single_chan,ctl_msg.num_adc_banks,ctl_msg.timer_period); */
    return 0;

}
