// file: rtfraw.cc
// by Michael Babcock, Steve Rosenbluth, 
// Jim Henson's Creature Shop
// compile with: g++ -Wall -O2 -o rtfraw rtfraw.cc

/*----------------------------------------------------------------
    This source file is subject to the Mozilla Public
    License Version 1.0 (the "License"); you may not use this file
    except in compliance with the License. You may obtain a copy of
    the License at http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS
    IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
    implied. See the License for the specific language governing
    rights and limitations under the License.
------------------------------------------------------------------*/


#include <iostream>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>                                 /* for exit */
#include "../daqcard700-config.h"                                                  /* defs for talking to data acq driver through rt-fifo */

#define DAQ_MAGIC 0x4DAC4DAC
#define SINGLE  0
#define ALL     1
#define DECIMAL 2
//#define PACKET_QTY 18

/*********************************/
int main (int argc, char *argv[]) 
{
    int in_file, i, numread;
    int mode;
    char dev_str[256] = {"/dev/rtf0"};
    
    mode = SINGLE;

    if (argc > 1)
    {
        if (strcmp(argv[1],"all") == 0)
            mode = ALL;
        else if (strcmp(argv[1],"byte") == 0)
            mode = DECIMAL;
        else if (strcmp(argv[1],"--help") == 0)
        {
            cout << "usage: rtfraw [all|byte] [0..3]\nno arg gives last conversion of first adc bank\n"
                 << "'all' gives all banks available\n'byte' gives each byte individually (in decimal)\n"
                 << "default fifo is 0, 2nd arg specifies another\n" ;
            exit(0);
        }

        if (argc > 2)
          switch (argv[2][0])
          {
            case '0':
                strcpy(dev_str,"/dev/rtf0");
            break;
            case '1':
                strcpy(dev_str,"/dev/rtf1");
            break;
            case '2':
                strcpy(dev_str,"/dev/rtf2");
            break;
            case '3':
                strcpy(dev_str,"/dev/rtf3");
            break;
          }
    }
    
    if ((in_file = open(dev_str, O_RDONLY|O_NONBLOCK)) < 0) {
        perror("");
        printf("Error opening %s\n",dev_str);
        exit(-1);
    }
    else
        printf("using %s.\n",dev_str);
    if (mode == SINGLE)
        cout << "Looking for last channel (15) of first (0) bank...\n";
    else if (mode == ALL)
        cout << "Looking for all channel data...\n";
    else if (mode == DECIMAL)
        cout << "Looking for individual bytes...\n";
    
    int buf[RESULT_PACKET_SIZE];
    unsigned char bytebuf[255];

    while (true) {
      if ((mode == SINGLE) || (mode == ALL))
      {
        if (read(in_file, buf, sizeof(int) * RESULT_PACKET_SIZE) == sizeof(int) * RESULT_PACKET_SIZE) 
        {
            if ((buf[0] != DAQ_MAGIC) && (mode != DECIMAL))
                cout << "Error: didn't see packet magic byte !\n";
            else
            {
                if (mode == SINGLE)
                {
                    if (buf[1] == 0)
                        cout << buf[RESULT_PACKET_SIZE-1] << endl;
                }
                else if (mode == ALL)
                {
                    for (i=2; i < RESULT_PACKET_SIZE; i++)
                    {
                    
                        cout << buf[i];
                        if (i < RESULT_PACKET_SIZE-1)
                            cout << " ";
                    }
                    cout << ".\n";
                }
            }
        }
      }
      else if (mode == DECIMAL)
      {
            numread = read(in_file, bytebuf, sizeof(unsigned char) * 255);
            if (numread > 0)
            {
                for (i=0; i < numread; i++)
                    printf("%u ",bytebuf[i]);
                cout << ".\n";
            }
      }
    }

    return 0;
}
